import java.util.*;
/**
 * ModArithmetic:
 * Implements static methods useful for modular arithmetic.
 *
 * Reference:
 * Trappe and Washington, "Introduction to Cryptography with Coding Theory"
 * 2006, 2nd edition
 */
public class ModArithmetic {
    
    /** Returns the greatest common denominator (GCD), 
     * computed using the Euclidian algorithm. */
    public static int gcd(int a, int b) {
        int small, big;
        int new_small, new_big;
        int remainder;
        int gcd;

        if(a > b) {
            big = a;
            small = b;
        } else {
            big = b;
            small = a;
        }

        while(true) {
            remainder = big % small;

            if(remainder==0) {
                gcd = small;
                break;
            }
            
            new_small = remainder;
            new_big = small;

            small = new_small;
            big = new_big;
        }        

        return gcd;

    }


    /** Solve ax + by = gcd(a,b) using extended Euclidian algorithm
     * and return x and y */
    public static void extendedEuclidian(int[] xy, int a, int b) {
        int small, big;
        int quotient, remainder;

        if(a > b) {
            big = a;
            small = b;
        } else {
            big = b;
            small = a;
        }

        ArrayList<Integer> x = new ArrayList<Integer>();
        ArrayList<Integer> y = new ArrayList<Integer>();
        ArrayList<Integer> q = new ArrayList<Integer>();
        ArrayList<Integer> r = new ArrayList<Integer>();

        x.add(0); x.add(1); // says the book
        y.add(1); y.add(0); // says the book
        q.add(0); r.add(0); // q and r start at 1

        int j = 2;
        int new_x, new_y;
        int new_small, new_big;
        while(true) { 

            quotient = big / small;
            remainder = big % small;

            q.add(quotient);
            r.add(remainder);

            // put the break here,
            // so we don't use the final quotient.
            // per the book.
            if(remainder == 0) {
                break;
            }

            new_x = -q.get(j-1)*x.get(j-1) + x.get(j-2);
            new_y = -q.get(j-1)*y.get(j-1) + y.get(j-2);

            x.add(new_x);
            y.add(new_y);

            new_small = remainder;
            new_big = small;

            small = new_small;
            big = new_big;

            j++;
        }

        xy[0] = x.get(x.size()-1);
        xy[1] = y.get(y.size()-1);
    }



    /** Returns solutions to ax = c (mod n) */
    public static ArrayList<Integer> solve(int a, int c, int n) { 
        ArrayList<Integer> result = new ArrayList<Integer>();

        int gcd = ModArithmetic.gcd(a,n);

        if( gcd==1 ) {
            // Case 1: gcd(a,n) = 1
            int inva = ModArithmetic.inverseMod(a,n);
            int rhs = (inva*c)%n;
            result.add(rhs);

        } else {
            // Case 2: gcd(a,n) > 1

            // if gcd does not divide c, no solution
            if( c%gcd > 0 ) {
                throw new RuntimeException("Error: gcd(a,n) = "+gcd+" is greater than 1 and does not divide c = "+c);
            }

            System.out.println("gcd(a,n) = "+gcd(a,n));

            // consider new congruence,
            // (a/gcd) x = (c/gcd)  (mod n/gcd)
            int newa = a/gcd;
            int newc = c/gcd;
            int newn = n/gcd;

            // now gcd(newa, newn) is 1
            System.out.println("Inverting "+newa+" mod "+newn);
            int invnewa = ModArithmetic.inverseMod(newa, newn);
            int x0 = (invnewa * newc) % newn;
            System.out.println("x0 = "+x0);
            for( int i=0; i<gcd; i++ ) {
                int xi = x0 + (i)*(n/gcd);
                result.add(xi);
            }
        }

        return result;

    }


    /** Returns the multiplicative inverse of a modulo n */
    public static int inverseMod(int a, int n) {
        int gcd = ModArithmetic.gcd(a,n);
        if( gcd > 1 ) {
            throw new RuntimeException("Error: gcd(a,n) must be 1 to invert a (mod n), but gcd(a,n) = "+gcd);
        }
        return inverseModEuclidian(a,n);
        //return inverseModEuler(a,n);
    }


    /** Returns the multiplicative inverse of a, mod n,
     * determined using the extended Euclidian algorithm. */
    public static int inverseModEuclidian(int a, int n) {
        // To find a^-1 (mod n):
        // 1. Use extended euclidian algorithm to find integers s and t such that
        //     as + nt = 1
        // 2. a^-1 = s (mod n)
        int[] xy = new int[2];
        extendedEuclidian(xy, a,n);
        // ensure inverse is positive
        if( xy[0] < 0 ) {
            xy[0] += n;
        }
        return xy[0];
    }


    /** Returns the multiplicative inverse of a, mod n, 
     * determined using Euler's totient function method,
     * a^{ phi(n) - 1 } = a^{-1} (mod n) */
    public static int inverseModEuler(int a, int n) {
        int tot, remainder, inv, inverse;

        inv = 1;
        tot = totient(n);
        for( int i=0; i<(tot-1); i++ ) {
            inv *= a;
            inv %= 26; // do this each step, otherwise things get messy
        }
        return inv;
    }


    /** Returns the value of Euler's totient function
     * (a count of the positive integers up to N that are 
     *  relatively prime to N). */
    public static int totient( int N ) { 
        int tot = 0;
        for( int c = N; c>=1; c-- ) {
            if( gcd(N,c)==1 ) {
                tot++; // c is relatively prime to N
            }
        }
        return tot;
    }

    /** Returns the value of x^a computed (mod n),
     * computed in a sane and efficient manner. */
    public static int exponentMod(int x, int a, int n) {
        int result = 1;
        for( int ia = 1; ia<=a; ia++ ) {
            result *= x;
            result %= n;
        }
        return result;
    }


    /** Bootstrap the prime factorization of n by providing one factor f */
    public static ArrayList<Integer> bootstrapFactorization( int n, int f ) { 
        ArrayList<Integer> result = new ArrayList<Integer>();
        int loopn, loopf, remainder;

        /////////////////////////////////////
        // This is a little half-baked.
        /////////////////////////////////////

        loopn = n; loopf = f;
        do {
            System.out.println("loopn = "+loopn+" loopf = "+loopf);
            if( loopn % loopf == 0 ) {
                result.add(loopf);
            }
            remainder = loopn / loopf;
            loopf = gcd(loopn, remainder);
            loopn = remainder;
        } while( loopf > 1 );

        // divide n by f
        // find gcd of n and result 

        return result;
    }


    public static ArrayList<Integer> fibonacci( int n ) {
        ArrayList<Integer> result = new ArrayList<Integer>();
        
        result.add(1); // F1
        result.add(1); // F2
        for(int i=2; i<n; i++) {
            int F_i = result.get(i-1) + result.get(i-2);
            result.add(F_i);
        }

        return result;
    }
}
